<?php
/* References
 * https://github.com/chipbennett/oenology/blob/master/functions/options.php#L84
 * http://www.chipbennett.net/2011/02/17/incorporating-the-settings-api-in-wordpress-themes/7/
 */
/* Add admin menu */
add_action( 'admin_menu', 'pm_lgate_admin_menu' );
function pm_lgate_admin_menu() {
	// Create options page
	$settings = add_options_page(PM_LGATE_PLUGIN_NAME, PM_LGATE_PLUGIN_NAME, 'manage_options', 'pm_lgate_settings', 'pm_lgate_options_page');
  	
  	// Load admin scripts when above page is called
  	add_action( 'load-'.$settings, 'pm_lgate_adminscripts' );
}

// Enqueue admin scripts
function pm_lgate_adminscripts() {
	$pm_lgate_opts = get_option( 'pm_lgate_options' );
	wp_enqueue_script( 'pm-lgate-admin', PM_LGATE_DIR.'modules/js/admin.js', array( 'jquery', 'wp-color-picker', 'media-upload', 'thickbox' ), PM_LGATE_VERSION, true );
	wp_enqueue_script( 'jquery-ui-accordion' );
	wp_enqueue_media();
	wp_localize_script( 'pm-lgate-admin', 'pm_lgate_object',
            array( 
				'nonce' => wp_create_nonce('pm_lgate_options'),
				'is_admin' => is_admin()
            	)
            );

	wp_enqueue_style( 'pm-lgate-admin', PM_LGATE_DIR.'modules/css/admin.css', '', PM_LGATE_VERSION, 'all' );
	wp_enqueue_style( 'wp-color-picker' );
}

// Output the settings page
function pm_lgate_options_page() {
	$tab = ( isset( $_GET['tab'] )  ? $_GET['tab'] : 'addforms' );
	$settings_section = 'pm_lgate_settings_'.$tab.'_tab';
	?>
	<div class="wrap">
		<h2><?php echo PM_LGATE_PLUGIN_NAME; ?> Settings</h2>
		<?php settings_errors( 'pm_lgate_options' ); ?>
		<?php pm_lgate_settings_page_tabs(); ?>

		<form action='options.php' method='post'>
			<?php 
			// Call functions to build form
			settings_fields( 'pm_lgate_options' );
			//do_settings_sections( 'pm_lgate_settings' );
			do_settings_sections( $settings_section );

			// Update settings based on tab			
			?>
			<p class="submit">
			<?php
			submit_button( 'Save Settings', 'primary', 'pm_lgate_options[submit-'.$tab.']', false, array( 'id' => 'submit-'.$tab ) );
			submit_button( 'Reset', 'secondary', 'pm_lgate_option[reset-'.$tab.']', false, array( 'id' => 'reset-'.$tab ) );
			?></p>
		</form>
	</div>
	<?php
}

// Create tabs
function pm_lgate_get_settings_page_tabs() {
	$tabs = array(
		'addforms' => 'Create Link Gates',
		'misc' => 'Misc.',
		);
	return $tabs;
}

// HTML Markup for tabs
function pm_lgate_settings_page_tabs( $current = 'addforms' ) {
	// Determind which tab to display
	if( isset( $_GET['tab'] ) ) :
		$current = $_GET['tab'];
	else:
		$current = 'addforms';
	endif;

	// Get list of tabs
	$tabs = pm_lgate_get_settings_page_tabs();
	$links = array();

	// Create links for each tab
	foreach ($tabs as $tab => $name) :
		if( $tab == $current ) :
			$links[] = '<a class="nav-tab nav-tab-active" href="?page=pm_lgate_settings&tab='.$tab.'">'.$name.'</a>';
		else :
			$links[] = '<a class="nav-tab" href="?page=pm_lgate_settings&tab='.$tab.'">'.$name.'</a>';
		endif;
	endforeach;

	echo '<h2 class="nav-tab-wrapper">';
	foreach ( $links as $link ) {
		echo $link;
	}
	echo '</h2>';
}

// Register settings
add_action( 'admin_init', 'pm_lgate_initialize_settings' );
function pm_lgate_initialize_settings() {
	// Register fields
	register_setting( 
		'pm_lgate_options',
		'pm_lgate_options', 
		'pm_lgate_validate_options' );

	// Add settings sections
	pm_lgate_add_settings_sections();
	//add_settings_section( 'one', 'One Section', 'pm_lgate_one_section_callback', 'pm_lgate_settings' );

	// Add settings fields
	pm_lgate_add_settings_fields();
	//add_settings_field( 'textfield', 'Text Field', 'pm_lgate_textfield_field_callback', 'pm_lgate_settings', 'one' );
}

/*---------------
 Sections
 ---------------*/
// Register multiple settings (called in admin_init)
function pm_lgate_add_settings_sections() {
	// Array with settings identifiers
	$sects = array(
		'addforms' => array(
			'id' => 'addforms',
			'title' => 'Gates',
			'page' => 'pm_lgate_settings_addforms_tab'
			),
		'housekeepings' => array(
			'id' => 'housekeepings',
			'title' => 'Plugin Housekeeping',
			'page' => 'pm_lgate_settings_misc_tab'
			),
		);

	// Loop through each
	foreach ($sects as $s) {
		add_settings_section( 
			$s['id'].'_section',
			$s['title'].' Options',
			'pm_lgate_'.$s['id'].'_section_callback',
			$s['page']
			);
	}
}

/*----------------
Sections callback
-----------------*/
// Section callback
function pm_lgate_addforms_section_callback() {
	echo '<p>Configure additional link gates. <span style="color: #C00403;">For best results, use your keyboard (Ctrl+V for Win and Cmd+V for Mac) to paste your form code.</span></p>';
}
function pm_lgate_housekeepings_section_callback() {
	echo '<p>Settings to keep the plugin and your site working well</p>';
}

/*------------------
Fields
------------------*/
// Register multiple fields (called in admin_init)
function pm_lgate_add_settings_fields () {
	// Option settings
	//$pm_lgate_opts = get_option('pm_lgate_options');
	// Array with field info
	$fields = array(
		'additionalForms' => array(
			'id' => 'additionalForms',
			'title' => 'Link Gates',
			'page' => 'pm_lgate_settings_addforms_tab',
			'section' => 'addforms_section',
			//'args' => array($pm_lgate_opts)
			),
		'delopt' => array(
			'id' => 'delopt',
			'title' => 'Delete all plugin data?',
			'page' => 'pm_lgate_settings_misc_tab',
			'section' => 'housekeepings_section'
			),
		);

	// Loop through each in array
	foreach ($fields as $f) {
		add_settings_field( 
			$f['id'],
			$f['title'],
			'pm_lgate_'.$f['id'].'_field_callback',
			$f['page'],
			$f['section']
			);
	}
}

/*--------------------
Fields callback
--------------------*/
function pm_lgate_additionalForms_field_callback($args) {
	//$forms = ( empty( $args[0]['add'] ) ) ? '' : $args[0]['add'] ;
	$forms = get_option('pm_lgate_options');
	$forms = $forms['add'];
	//print_r($forms);
	?>
	<div id="add_form_list">
		<?php if( !empty( $forms ) ) {
			$saved_forms = new PmillFormOptions( $forms );
			echo $saved_forms->pm_optin_form_options();
			} else { ?>
			<?php
			$formOptions = array(
				'1' => array(
			  		'count' => '1',
			  		'optkey' => 'add',
					'headline' => '',
					'desc' => '',
					'cover' =>  '',
					'coveralign' =>  'l',
					'size' =>  'regular',
					'borderw' => '0',
					'borderstyle' => 'none',
					'bordercolor' => '',
					'bgcolor' => '',
					'bgimage' => '',
					'bghandling' => 'no-repeat',
					'txt_color' => '',
					'radius' => '0',
					'ar_code' =>  '',
					'ar_name' => '',
					'ar_email' => '',
					'ar_url' => '',
					'ar_hidden' => '',
					'emailinputText' => 'Your email',
					'nameinputText' => 'Your name',
					'button_text' => 'Subscribe',
					'button_text_color' => '#FFFFFF',
			  		'button_color' => '#CC0403'
					)
				);
			$empty_form = new PmillFormOptions( $formOptions );
			echo $empty_form->pm_optin_form_options();
			?>
			<?php }
		?>
	</div>
	<button class="add_a_form" class="secondary-button">Add A Link Gate</button>
	<?php
}
function pm_lgate_delopt_field_callback() {	
	$pm_lgate_opts = get_option( 'pm_lgate_options' );
	$del_opt_settings = ( empty( $pm_lgate_opts['del_opt_settings'] ) ) ? '' : $pm_lgate_opts['del_opt_settings'];
	?>
	<input id="del_opt_settings" type="checkbox" name="pm_lgate_options[del_opt_settings]" value="1" id="del_opt_settings" <?php checked( $del_opt_settings, '1', true ); ?>> <label for="del_opt_settings">Yes. Delete all plugin data when I uninstall</label><br>
	<small>Use with caution! When checked, all of this plugin's settings will be deleted on plugin uninstall. Disabling the plugin will not delete data only uninstall.</small>
	<?php
}

// Validate options
function pm_lgate_validate_options( $input ) {
	$pm_lgate_opts = get_option( 'pm_lgate_options' );
	$valid_input = $pm_lgate_opts;

	// Determine which tab is submitted
	$misc = ( !empty( $input['submit-misc'] ) ? true : false );
	$addforms = ( !empty( $input['submit-addforms'] ) ? true : false );

	// Valid radio arrays
	$size_opts = array('regular', 'mega');
	$borderstyles = array('none', 'solid', 'dotted', 'dashed', 'double');
	$bghandling_opts = array('no-repeat', 'repeat', 'repeat-x', 'repeat-y', 'cover');
	$allowed_form_input = array(
		'form' => array(
			'action' => true,
			'accept' => true,
			'accept-charset' => true,
			'enctype' => true,
			'method' => true,
			'name' => true,
			'target' => true,
			),
		'input' => array(
			'name' => true,
			'type' => true,
			'value' => true
			)
		);

	if( $misc ) {
		// validate only misc settings
		$valid_del_opt_settings = ( isset( $input['del_opt_settings'] ) && $input['del_opt_settings'] == '1' ) ? '1' : '0' ;

		// Use saved settings for options not being validated
		$add = ( empty( $pm_lgate_opts['add'] ) ) ? '' : $pm_lgate_opts['add'] ;

		// Build array to save
		$valid_input = array(
		'add' => $add,
		'del_opt_settings' => $valid_del_opt_settings
		);
	} elseif ( $addforms ) {
		// validate only those settings
		$additional_forms = ( empty( $input['add'] ) ) ? '' : $input['add'] ;

		// Check to see if data is as expected
		if( is_array( $additional_forms ) ) {
			// Loop through & validate each setting
			foreach ($additional_forms as $form_key => $form) {
				$valid_add_forms[$form_key]['headline'] = ( empty( $form['headline'] ) ) ? '' : sanitize_text_field( $form['headline'] ) ;
				$valid_add_forms[$form_key]['desc'] = ( isset( $form['desc'] ) ) ? wp_kses_post( $form['desc'] ) : '';
				$valid_add_forms[$form_key]['cover'] = ( isset( $form['cover'] ) ) ? esc_url_raw($form['cover']) : '' ;
				$valid_add_forms[$form_key]['coveralign'] = ( isset($form['coveralign']) && in_array($form['coveralign'], array('l', 'r') ) ) ? $form['coveralign'] : '' ;
				$valid_add_forms[$form_key]['size'] = ( isset($form['size']) && in_array($form['size'], $size_opts) ) ? $form['size'] : '' ;
				$valid_add_forms[$form_key]['borderw'] = ( !empty( $form['borderw'] ) && is_numeric( $form['borderw'] ) ) ?  sanitize_text_field( $form['borderw'] ) : '' ;
				$valid_add_forms[$form_key]['borderstyle'] = ( isset($form['borderstyle']) && in_array($form['borderstyle'], $borderstyles) ) ? $form['borderstyle'] : '' ;
				$valid_add_forms[$form_key]['bordercolor'] = ( isset( $form['bordercolor'] ) && ( preg_match( '/^#[a-f0-9]{6}$/i', $form['bordercolor'] ) == '1' ) ) ? $form['bordercolor'] : '' ;
				$valid_add_forms[$form_key]['bgcolor'] = ( isset( $form['bgcolor'] ) && ( preg_match( '/^#[a-f0-9]{6}$/i', $form['bgcolor'] ) == '1' ) ) ? $form['bgcolor'] : '' ;
				$valid_add_forms[$form_key]['bgimage'] = ( isset( $form['bgimage'] ) ) ? esc_url_raw( $form['bgimage'] ) : '' ;
				$valid_add_forms[$form_key]['bghandling'] = ( isset($form['bghandling']) && in_array($form['bghandling'], $bghandling_opts) ) ? $form['bghandling'] : '' ;
				$valid_add_forms[$form_key]['txt_color'] = ( isset( $form['txt_color'] ) && ( preg_match( '/^#[a-f0-9]{6}$/i', $form['txt_color'] ) == '1' ) ) ? $form['txt_color'] : '' ;
				$valid_add_forms[$form_key]['radius'] = ( !empty( $form['radius'] ) && is_numeric( $form['radius'] ) ) ?  sanitize_text_field( $form['radius'] ) : ''  ;
				$valid_add_forms[$form_key]['ar_code'] = ( isset( $form['ar_code'] ) ) ? wp_kses( $form['ar_code'], $allowed_form_input ) : '';
				$valid_add_forms[$form_key]['ar_name'] = ( ctype_alnum( $form['ar_name'] ) == 1 ) ? $form['ar_name'] : '' ;
				$valid_add_forms[$form_key]['ar_name'] = ( preg_match("/^[A-Za-z0-9_.-]+$/",$form['ar_name']) == 1 ) ? $form['ar_name'] : '';
				$valid_add_forms[$form_key]['ar_email'] = ( preg_match("/^[A-Za-z0-9_.-]+$/",$form['ar_email']) == 1 ) ? $form['ar_email'] : '';
				$valid_add_forms[$form_key]['ar_url'] = ( isset( $form['ar_url'] ) ) ? esc_url_raw( $form['ar_url'] ) : '' ;
				$valid_add_forms[$form_key]['ar_hidden'] = ( isset( $form['ar_hidden'] ) ) ? $form['ar_hidden'] : '' ;				
				$valid_add_forms[$form_key]['emailinputText'] = ( isset( $form['emailinputText'] ) ) ? sanitize_text_field( $form['emailinputText'] ) : 'Primary email address' ;
				$valid_add_forms[$form_key]['nameinputText'] = ( isset( $form['nameinputText'] ) ) ? sanitize_text_field( $form['nameinputText'] ) : 'Name' ;
				$valid_add_forms[$form_key]['button_text'] = ( isset( $form['button_text'] ) ) ? sanitize_text_field( $form['button_text'] ) : 'Subscribe' ;
				$valid_add_forms[$form_key]['button_text_color'] = ( isset( $form['button_text_color'] ) && ( preg_match( '/^#[a-f0-9]{6}$/i', $form['button_text_color'] ) == '1' ) ) ? $form['button_text_color'] : '' ;
				$valid_add_forms[$form_key]['button_color'] = ( isset( $form['button_color'] ) && ( preg_match( '/^#[a-f0-9]{6}$/i', $form['button_color'] ) == '1' ) ) ? $form['button_color'] : '' ;
			}
		} else {
			$valid_add_forms = '';
		}

		// Build array to save
		$valid_input = array(
		'add' => $valid_add_forms,
		'del_opt_settings' => $pm_lgate_opts['del_opt_settings']
		);
	}

	return $valid_input;
	//print_r($valid_input);
	//exit(); // For testing
}
?>